import os
import torch

from .core import *
from .utils import set_default_env_vars
from .utils.options import CompileOptions
from .utils.options import _GLOBAL_compile_options as compile_options


set_default_env_vars()



import dataclasses
import torch
from .registry import register_backend


def make_inductor_options(options):
    inductor_options = {}
    if options is None:
        return inductor_options
    for filed in dataclasses.fields(options):
        filed_name = filed.name
        inductor_options[f"inductor.{filed_name}"] = getattr(options, filed_name)
    return inductor_options


@register_backend("nexfort")
def compile(torch_module: torch.nn.Module, *, options=None):
    from nexfort.utils.memory_format import apply_memory_format
    from nexfort.compilers import nexfort_compile
    from ..nexfort.deployable_module import NexfortDeployableModule
    from ..utils import CompileOptions

    options = options if options is not None else CompileOptions()
    nexfort_options = options.nexfort
    if nexfort_options.memory_format != torch.preserve_format:
        model = apply_memory_format(
            torch_module, memory_format=nexfort_options.memory_format
        )
    model = nexfort_compile(
        model, options=make_inductor_options(nexfort_options.inductor)
    )
    return NexfortDeployableModule(model)



import types
import torch
import oneflow as flow

from ..core.deployable_module import DeployableModule
from ..transform.manager import transform_mgr
from ..utils.oneflow_exec_mode import oneflow_exec_mode, oneflow_exec_mode_enabled
from ..utils.args_tree_util import input_output_processor
from ..utils.log_utils import logger
from ..utils.param_utils import parse_device, check_device, generate_constant_folding_info
from ..utils.graph_management_utils import graph_file_management
from ..utils.online_quantization_utils import quantize_and_deploy_wrapper
from ..utils.options import OneflowCompileOptions

from .utils import handle_deployable_exception, get_mixed_dual_module, get_oneflow_graph


class OneflowDeployableModule(DeployableModule):
    def __init__(
        self, torch_module, oneflow_module, dynamic=True, options=None,
    ):
        torch.nn.Module.__init__(self)
        object.__setattr__(
            self,
            "_deployable_module_model",
            get_mixed_dual_module(torch_module.__class__)(torch_module, oneflow_module),
        )
        object.__setattr__(self, "_modules", torch_module._modules)
        object.__setattr__(self, "_torch_module", torch_module)
        self._deployable_module_enable_dynamic = dynamic
        self._deployable_module_quant_config = None
        self._deployable_module_options = (
            options if options is not None else OneflowCompileOptions()
        )
        self._deployable_module_dpl_graph = None
        self._is_raw_deployable_module = True
        self._load_graph_first_run = True

    @classmethod
    def from_existing(cls, existing_module, dynamic=True, options=None):
        torch_module = existing_module._deployable_module_model._torch_module
        oneflow_module = existing_module._deployable_module_model._oneflow_module
        instance = cls(torch_module, oneflow_module, dynamic, options)
        instance._deployable_module_dpl_graph = None
        if hasattr(existing_module, "_deployable_module_dpl_graph"):
            instance._deployable_module_dpl_graph = (
                existing_module._deployable_module_dpl_graph
            )
        instance._load_graph_first_run = existing_module._load_graph_first_run
        instance._deployable_module_input_count = (
            existing_module._deployable_module_input_count
        )
        instance._deployable_module_quant_config = (
            existing_module._deployable_module_quant_config
        )

        return instance

    def get_graph(self):
        if self._deployable_module_dpl_graph is not None:
            return self._deployable_module_dpl_graph
        self._deployable_module_dpl_graph = get_oneflow_graph(
            self._deployable_module_model.oneflow_module,
            self._deployable_module_options.max_cached_graph_size,
            self._deployable_module_enable_dynamic,
        )
        # Enable debug mode
        if transform_mgr.debug_mode:
            self._deployable_module_dpl_graph.debug(0)
        if self._deployable_module_options.debug_level > 0:
            self._deployable_module_dpl_graph.debug(
                self._deployable_module_options.debug_level
            )
        return self._deployable_module_dpl_graph

    @input_output_processor
    @handle_deployable_exception
    @graph_file_management
    def apply_model(self, *args, **kwargs):
        if self._deployable_module_options.use_graph:
            dpl_graph = self.get_graph()
            with oneflow_exec_mode():
                output = dpl_graph(*args, **kwargs)
        else:
            with oneflow_exec_mode():
                output = self._deployable_module_model.oneflow_module.apply_model(
                    *args, **kwargs
                )
        return output

    @quantize_and_deploy_wrapper
    @input_output_processor
    @handle_deployable_exception
    @graph_file_management
    def forward(self, *args, **kwargs):
        if self._deployable_module_options.use_graph:
            dpl_graph = self.get_graph()
            with oneflow_exec_mode():
                output = dpl_graph(*args, **kwargs)
        else:
            with oneflow_exec_mode():
                output = self._deployable_module_model.oneflow_module(*args, **kwargs)
        return output

    def to(self, *args, **kwargs):
        if self._deployable_module_dpl_graph is None:
            self._deployable_module_model.to(*args, **kwargs)
            return self

        # assert the target device is same as graph device
        target_device = parse_device(args, kwargs)
        if (
            target_device is not None
            and len(self._deployable_module_dpl_graph._blocks) > 0
        ):
            current_device = next(self._deployable_module_dpl_graph._state()).device
            if not check_device(current_device, target_device):
                raise RuntimeError(
                    f"After graph built, the device of graph can't be modified, current device: {current_device}, target device: {target_device}"
                )
        self._deployable_module_model.to(*args, **kwargs)
        return self

    # TODO(): Just for transformers VAE decoder
    @input_output_processor
    @handle_deployable_exception
    @graph_file_management
    def decode(self, *args, **kwargs):
        if self._deployable_module_options.use_graph:

            def _build(graph, *args, **kwargs):
                return graph.model.decode(*args, **kwargs)

            dpl_graph = self.get_graph()
            dpl_graph.build = types.MethodType(_build, dpl_graph)
            with oneflow_exec_mode():
                output = dpl_graph(*args, **kwargs)
        else:
            with oneflow_exec_mode():
                output = self._deployable_module_model.oneflow_module.decode(
                    *args, **kwargs
                )
        return output

    def __getattr__(self, name):
        return getattr(self._deployable_module_model, name)

    def load_graph(self, file_path, device=None, run_warmup=True, *, state_dict=None):
        self.get_graph().load_graph(
            file_path, device, run_warmup, state_dict=state_dict
        )
        generate_constant_folding_info(self)

    def save_graph(self, file_path, *, process_state_dict=lambda x: x):
        self.get_graph().save_graph(file_path, process_state_dict=process_state_dict)

    def extra_repr(self) -> str:
        return self._deployable_module_model.extra_repr()

    def set_graph_file(self, file_path: str) -> None:
        """ Sets the path of the graph file.

        If the new file path is different from the old one, clears old graph data.

        Args:
            `file_path` (str): The path of the graph file.
        """
        old_file_path = self.get_graph_file()
        if file_path and old_file_path == file_path:
            return
        self._deployable_module_options.graph_file = file_path
        self._clear_old_graph()

    def _clear_old_graph(self):
        self._load_graph_first_run = True
        self._deployable_module_dpl_graph = None
        del self._deployable_module_model.oneflow_module

    def get_graph_file(self):
        return self._deployable_module_options.graph_file

    def apply_online_quant(self, quant_config):
        """
        Applies the provided quantization configuration for online use.

        Args:
            quant_config (QuantizationConfig): The quantization configuration to apply.

        Example:
            >>> from onediff_quant.quantization import QuantizationConfig
            >>> quant_config = QuantizationConfig.from_settings(
            ...     quantize_conv=True,
            ...     quantize_linear=True,
            ...     conv_mae_threshold=0.005,
            ...     linear_mae_threshold=0.005,
            ...     conv_compute_density_threshold=300,
            ...     linear_compute_density_threshold=100,
            ...     cache_dir=args.cache_dir)
            >>> model.apply_online_quant(quant_config)
        """
        self._deployable_module_quant_config = quant_config



import os
import types
from typing import Any
from itertools import chain

import torch
import oneflow as flow
from oneflow.utils.tensor import to_torch

from ..transform.builtin_transform import torch2oflow
from ..utils.oneflow_exec_mode import oneflow_exec_mode, oneflow_exec_mode_enabled
from ..utils.log_utils import logger


class DualModule(torch.nn.Module):
    def __init__(self, torch_module, oneflow_module):
        torch.nn.Module.__init__(self)
        object.__setattr__(self, "_torch_module", torch_module)
        object.__setattr__(self, "_oneflow_module", oneflow_module)
        object.__setattr__(self, "_modules", torch_module._modules)
        object.__setattr__(self, "_parameters", torch_module._parameters)
        object.__setattr__(self, "_buffers", torch_module._buffers)

    @property
    def oneflow_module(self):
        if self._oneflow_module is not None:
            return self._oneflow_module

        logger.debug(f"Convert {type(self._torch_module)} ...")
        self._oneflow_module = torch2oflow(self._torch_module)
        logger.debug(f"Convert {type(self._torch_module)} done!")
        return self._oneflow_module

    @oneflow_module.deleter
    def oneflow_module(self):
        if self._oneflow_module:
            del self._oneflow_module
            setattr(self, "_oneflow_module", None)

    def to(self, *args, **kwargs):
        if oneflow_exec_mode_enabled():
            self._oneflow_module.to(*args, **kwargs)
        else:
            if self._oneflow_module is not None:
                of_args = [torch2oflow(v) for v in args]
                of_kwargs = {k: torch2oflow(v) for k, v in kwargs.items()}
                self._oneflow_module.to(*of_args, **of_kwargs)
                self._torch_module_to_with_check(*args, **kwargs)
            else:
                self._torch_module.to(*args, **kwargs)

    def _torch_module_to_with_check(self, *args, **kwargs):
        def _align_tensor(torch_module, oneflow_module):
            oneflow_tensor_list = set(
                [x for x, _ in oneflow_module.named_parameters()]
                + [x for x, _ in oneflow_module.named_buffers()]
            )
            for name, tensor in chain.from_iterable(
                [torch_module.named_parameters(), torch_module.named_buffers(),]
            ):
                if name not in oneflow_tensor_list:
                    tensor.data = tensor.to(*args, **kwargs)
                else:
                    oneflow_tensor = oneflow_module.get_parameter(name)
                    if oneflow_tensor is None:
                        tensor.data = tensor.to(*args, **kwargs)
                    elif tensor.data_ptr() != oneflow_tensor.data_ptr():
                        tensor.data = to_torch(oneflow_tensor.data)

        oneflow_module_list = set([x for x, _ in self._oneflow_module.named_modules()])
        for name, module in self._torch_module.named_modules():
            if name not in oneflow_module_list:
                module.to(*args, **kwargs)
            else:
                _align_tensor(module, self._oneflow_module.get_submodule(name))

    def __getattr__(self, name):
        if name == "_torch_module" or name == "_oneflow_module":
            return super().__getattribute__(name)

        torch_attr = getattr(self._torch_module, name)
        oneflow_attr = (
            None
            if self._oneflow_module is None
            else getattr(self._oneflow_module, name)
        )

        if isinstance(torch_attr, torch.nn.ModuleList):
            if oneflow_attr is None:
                oneflow_attr = flow.nn.ModuleList([None] * len(torch_attr))
            return DualModuleList(torch_attr, oneflow_attr)

        elif isinstance(torch_attr, torch.nn.Module):
            from .utils import get_mixed_dual_module

            return get_mixed_dual_module(torch_attr.__class__)(torch_attr, oneflow_attr)
        else:
            return oneflow_attr if oneflow_exec_mode_enabled() else torch_attr

    def __setattr__(self, name: str, value: Any) -> None:
        if name in ["_torch_module", "_oneflow_module"]:
            super().__setattr__(name, value)
        else:  # TODO: aviod memory up when set attr
            if self._oneflow_module is not None:
                v = torch2oflow(value)
                if isinstance(v, flow.Tensor):
                    obj = getattr(self._oneflow_module, name)
                    obj.copy_(v)
                else:
                    setattr(self._oneflow_module, name, v)
            setattr(self._torch_module, name, value)

    def extra_repr(self) -> str:
        return self._torch_module.extra_repr()


class DualModuleList(torch.nn.ModuleList):
    def __init__(self, torch_modules, oneflow_modules):
        super().__init__()
        assert len(torch_modules) == len(oneflow_modules)
        self._torch_modules = torch_modules
        self._oneflow_modules = oneflow_modules
        from .utils import get_mixed_dual_module

        dual_modules = []
        for torch_module, oneflow_module in zip(
            self._torch_modules, self._oneflow_modules
        ):
            dual_modules.append(
                get_mixed_dual_module(torch_module.__class__)(
                    torch_module, oneflow_module
                )
            )
        # clear self._modules since `self._torch_modules = torch_modules` will append a module to self._modules
        self._modules.clear()
        self += dual_modules

    def __setitem__(self, idx: int, module: DualModule):
        idx = self._get_abs_string_index(idx)
        setattr(self._torch_modules, str(idx), module._torch_module)
        setattr(self._oneflow_modules, str(idx), module._oneflow_module)
        return setattr(self, str(idx), module)

    def __setattr__(self, key, value):
        if key in ("_torch_modules", "_oneflow_modules"):
            return object.__setattr__(self, key, value)
        if isinstance(value, DualModule):
            setattr(self._torch_modules, key, value._torch_module)
            setattr(self._oneflow_modules, key, value._oneflow_module)
        else:
            setattr(self._torch_modules, key, value)
            value = torch2oflow(value)
            setattr(self._oneflow_modules, key, value)
        return object.__setattr__(self, key, value)



import dataclasses
from typing import Dict
import torch


@dataclasses.dataclass
class OneflowCompileOptions:
    use_graph: bool = True
    debug_level: int = -1
    max_cached_graph_size: int = 9
    graph_file: str = None
    graph_file_device: torch.device = None

    # Optimization related environment variables
    run_graph_by_vm: bool = None
    graph_delay_variable_op_execution: bool = None

    conv_allow_half_precision_accumulation: bool = None
    matmul_allow_half_precision_accumulation: bool = None
    attention_allow_half_precision_accumulation: bool = None
    attention_allow_half_precision_score_accumulation_max_m: int = None
    attention_allow_quantization: bool = None

    mlir_cse: bool = None
    mlir_enable_inference_optimization: bool = None
    mlir_enable_round_trip: bool = None
    mlir_fuse_forward_ops: bool = None
    mlir_fuse_ops_with_backward_impl: bool = None
    mlir_group_matmul: bool = None
    mlir_prefer_nhwc: bool = None
    mlir_fuse_kernel_launch: bool = None

    kernel_enable_cuda_graph: bool = None
    kernel_enable_fused_conv_bias: bool = None
    kernel_enable_fused_linear: bool = None
    kernel_conv_cutlass_impl_enable_tuning_warmup: bool = None
    kernel_enable_conv2d_tuning_warmup: bool = None
    kernel_gemm_cutlass_impl_enable_tuning_warmup: bool = None
    kernel_conv_enable_cutlass_impl: bool = None
    kernel_gemm_enable_cutlass_impl: bool = None
    kernel_glu_enable_dual_gemm_impl: bool = None
    kernel_glu_enable_y_gemm_impl: bool = None
    kernel_glu_quant_enable_dual_gemm_impl: bool = None


@dataclasses.dataclass
class NexfortInductorCompileOptions:
    disable: bool = False
    mode: str = None
    options: Dict = dataclasses.field(default_factory=dict)


@dataclasses.dataclass
class NexfortCompileOptions:
    memory_format: torch.memory_format
    fuse_qkv_projections: bool
    inductor: NexfortInductorCompileOptions

    def __init__(
        self,
        memory_format=torch.channels_last,
        fuse_qkv_projections=True,
        inductor=None,
    ):
        if isinstance(memory_format, str):
            memory_format = getattr(torch, memory_format)
        self.memory_format = memory_format
        self.fuse_qkv_projections = fuse_qkv_projections
        self.inductor = (
            inductor if inductor is not None else NexfortInductorCompileOptions()
        )


@dataclasses.dataclass
class CompileOptions:
    # common options
    dynamic: bool

    # oneflow specific options
    oneflow: OneflowCompileOptions

    # nexfort specific options
    nexfort: NexfortCompileOptions

    def __init__(self, dynamic=True, oneflow=None, nexfort=None):
        self.dynamic = dynamic
        self.oneflow = oneflow if oneflow is not None else OneflowCompileOptions()
        self.nexfort = nexfort if nexfort is not None else NexfortCompileOptions()


# a global default compile options
_GLOBAL_compile_options = CompileOptions()



import torch
from .registry import register_backend


@register_backend("oneflow")
def compile(torch_module: torch.nn.Module, *, options=None):
    """
    Transform a torch nn.Module to oneflow.nn.Module, then optimize it with oneflow.nn.Graph.
    Args:
       model (torch.nn.Module): Module to optimize
       options (CompileOptions): Compilation options to pass to the compiler:
        - 'dynamic': When this is True, we will generate one graph and reuse it to avoid recompilations when
                     input shape change.  This may not always work as some operations/optimizations break the contition of
                     reusing. When this is False, we will generate a graph for each new input shape, and will always specialize.
                     default True.
        - 'use_graph' whether to optimize with oneflow.nn.Graph, default True.
        - 'debug' which config the nn.Graph debug level, default -1(no debug info), max 3(max debug info).
        - 'size' which config the cache size when cache is enabled. Note that after onediff v0.12, cache is default disabled.
        - 'graph_file' (None) generates a compilation cache file. If the file exists, loading occurs; if not, the compilation result is saved after the first run.
        - 'graph_file_device' (None) sets the device for the graph file, default None.  If set, the compilation result will be converted to the specified device.
    """
    from ..oneflow.deployable_module import OneflowDeployableModule
    from ..oneflow.utils import get_mixed_deployable_module
    from ..transform.custom_transform import set_default_registry
    from ..utils import CompileOptions, set_oneflow_env_vars
    from ..utils.param_utils import (
        state_update_hook,
        init_state_update_attr,
        forward_pre_check_and_update_state_hook,
        forward_generate_constant_folding_info_hook,
    )

    set_default_registry()

    options = options if options is not None else CompileOptions()
    set_oneflow_env_vars(options.oneflow)

    def wrap_module(module):
        if isinstance(module, OneflowDeployableModule):
            assert not module._is_raw_deployable_module
            return module.__class__.from_existing(
                module, options.dynamic, options.oneflow
            )
        else:
            return get_mixed_deployable_module(module.__class__)(
                module, None, options.dynamic, options.oneflow
            )

    model = wrap_module(torch_module)
    assert isinstance(model, OneflowDeployableModule)
    assert isinstance(model, torch_module.__class__)

    def state_dict_hook(module, state_dict, prefix, local_metadata):
        pytorch_key_prefix = "_deployable_module_model._torch_module."
        new_state_dict = type(state_dict)()
        for k, v in state_dict.items():
            # _deployable_module_model._torch_module.out.2.weight => out.2.weight
            if k.startswith(pytorch_key_prefix):
                new_k = k[len(pytorch_key_prefix) :]
                new_state_dict[new_k] = v
            else:
                new_state_dict[k] = v
        return new_state_dict

    model._register_state_dict_hook(state_dict_hook)

    # for checking state dict update of torch_module
    model._torch_module.register_load_state_dict_post_hook(state_update_hook)
    init_state_update_attr(model._torch_module)

    # hooks for constant folding
    model.register_forward_pre_hook(forward_pre_check_and_update_state_hook)
    model.register_forward_hook(forward_generate_constant_folding_info_hook)

    return model



import dataclasses
import os
from typing import Optional


def parse_boolean_from_env(env_var, default_value=None):
    env_var = os.getenv(env_var)
    if env_var is None:
        return default_value
    env_var = env_var.lower()
    return env_var in ("1", "true", "yes", "on", "y")


def set_boolean_env_var(env_var: str, val: Optional[bool]):
    if val is None:
        os.environ.pop(env_var, None)
    else:
        os.environ[env_var] = "1" if val else "0"


def parse_integer_from_env(env_var, default_value=None):
    env_var = os.getenv(env_var)
    if env_var is None:
        return default_value
    return int(env_var)


def set_integer_env_var(env_var: str, val: Optional[int]):
    if val is None:
        os.environ.pop(env_var, None)
    else:
        os.environ[env_var] = str(int(val))


def _set_env_vars(field2env_var, options):
    for field in dataclasses.fields(options):
        field_name = field.name
        field_value = getattr(options, field_name)
        if field_value is None or field_name not in field2env_var:
            continue
        env_var = field2env_var[field_name]
        set_env_var = None
        if field.type in (bool, Optional[bool]):
            set_env_var = set_boolean_env_var
        elif field.type in (int, Optional[int]):
            set_env_var = set_integer_env_var
        else:
            raise ValueError(f"Unsupported type {field.type}")
        set_env_var(env_var, field_value)


def set_oneflow_env_vars(options):
    field2env_var = {
        "run_graph_by_vm": "ONEFLOW_RUN_GRAPH_BY_VM",
        "graph_delay_variable_op_execution": "ONEFLOW_GRAPH_DELAY_VARIABLE_OP_EXECUTION",
        "mlir_cse": "ONEFLOW_MLIR_CSE",
        "mlir_enable_inference_optimization": "ONEFLOW_MLIR_ENABLE_INFERENCE_OPTIMIZATION",
        "mlir_enable_round_trip": "ONEFLOW_MLIR_ENABLE_ROUND_TRIP",
        "mlir_fuse_forward_ops": "ONEFLOW_MLIR_FUSE_FORWARD_OPS",
        "mlir_fuse_ops_with_backward_impl": "ONEFLOW_MLIR_FUSE_OPS_WITH_BACKWARD_IMPL",
        "mlir_group_matmul": "ONEFLOW_MLIR_GROUP_MATMUL",
        "mlir_prefer_nhwc": "ONEFLOW_MLIR_PREFER_NHWC",
        "mlir_fuse_kernel_launch": "ONEFLOW_MLIR_FUSE_KERNEL_LAUNCH",
        "kernel_enable_cuda_graph": "ONEFLOW_KERNEL_ENABLE_CUDA_GRAPH",
        "kernel_enable_fused_conv_bias": "ONEFLOW_KERNEL_ENABLE_FUSED_CONV_BIAS",
        "kernel_enable_fused_linear": "ONEFLOW_KERNEL_ENABLE_FUSED_LINEAR",
        "kernel_conv_cutlass_impl_enable_tuning_warmup": "ONEFLOW_KERNEL_CONV_CUTLASS_IMPL_ENABLE_TUNING_WARMUP",
        "kernel_gemm_cutlass_impl_enable_tuning_warmup": "ONEFLOW_KERNEL_GEMM_CUTLASS_IMPL_ENABLE_TUNING_WARMUP",
        "kernel_conv_enable_cutlass_impl": "ONEFLOW_KERNEL_CONV_ENABLE_CUTLASS_IMPL",
        "kernel_enable_conv2d_tuning_warmup": "ONEFLOW_CONV2D_KERNEL_ENABLE_TUNING_WARMUP",
        "kernel_gemm_enable_cutlass_impl": "ONEFLOW_KERNEL_GEMM_ENABLE_CUTLASS_IMPL",
        "kernel_glu_enable_dual_gemm_impl": "ONEFLOW_KERNEL_GLU_ENABLE_DUAL_GEMM_IMPL",
        "kernel_glu_enable_y_gemm_impl": "ONEFLOW_KERNEL_GLU_ENABLE_Y_GEMM_IMPL",
        "kernel_glu_quant_enable_dual_gemm_impl": "ONEFLOW_KERNEL_GLU_QUANT_ENABLE_DUAL_GEMM_IMPL",
        "conv_allow_half_precision_accumulation": "ONEFLOW_CONV_ALLOW_HALF_PRECISION_ACCUMULATION",
        "matmul_allow_half_precision_accumulation": "ONEFLOW_MATMUL_ALLOW_HALF_PRECISION_ACCUMULATION",
        "attention_allow_half_precision_accumulation": "ONEFLOW_ATTENTION_ALLOW_HALF_PRECISION_ACCUMULATION",
        "attention_allow_half_precision_score_accumulation_max_m": "ONEFLOW_ATTENTION_ALLOW_HALF_PRECISION_SCORE_ACCUMULATION_MAX_M",
    }
    _set_env_vars(field2env_var, options)


def set_oneflow_default_env_vars():
    # ONEFLOW_RUN_GRAPH_BY_VM must set here to enable nn.Graph init with vm run
    os.environ.setdefault("ONEFLOW_RUN_GRAPH_BY_VM", "1")
    os.environ.setdefault("ONEFLOW_GRAPH_DELAY_VARIABLE_OP_EXECUTION", "1")

    os.environ.setdefault("ONEFLOW_MLIR_CSE", "1")
    os.environ.setdefault("ONEFLOW_MLIR_ENABLE_INFERENCE_OPTIMIZATION", "1")
    os.environ.setdefault("ONEFLOW_MLIR_ENABLE_ROUND_TRIP", "1")
    os.environ.setdefault("ONEFLOW_MLIR_FUSE_FORWARD_OPS", "1")
    os.environ.setdefault("ONEFLOW_MLIR_FUSE_OPS_WITH_BACKWARD_IMPL", "1")
    os.environ.setdefault("ONEFLOW_MLIR_GROUP_MATMUL", "1")
    os.environ.setdefault("ONEFLOW_MLIR_PREFER_NHWC", "1")

    os.environ.setdefault("ONEFLOW_KERNEL_ENABLE_FUSED_CONV_BIAS", "1")
    os.environ.setdefault("ONEFLOW_KERNEL_ENABLE_FUSED_LINEAR", "1")
    os.environ.setdefault("ONEFLOW_KERNEL_CONV_CUTLASS_IMPL_ENABLE_TUNING_WARMUP", "1")
    os.environ.setdefault("ONEFLOW_KERNEL_GEMM_CUTLASS_IMPL_ENABLE_TUNING_WARMUP", "1")
    os.environ.setdefault("ONEFLOW_KERNEL_CONV_ENABLE_CUTLASS_IMPL", "1")
    os.environ.setdefault("ONEFLOW_KERNEL_GEMM_ENABLE_CUTLASS_IMPL", "1")
    os.environ.setdefault("ONEFLOW_CONVOLUTION_BIAS_ADD_ACT_FUSION", "1")
    # os.environ.setdefault("ONEFLOW_KERNEL_GLU_ENABLE_DUAL_GEMM_IMPL", "0")
    # os.environ.setdefault("ONEFLOW_KERNEL_GLU_ENABLE_Y_GEMM_IMPL", "0")
    # os.environ.setdefault("ONEFLOW_KERNEL_GLU_QUANT_ENABLE_DUAL_GEMM_IMPL", "0")

    os.environ.setdefault("ONEFLOW_CONV_ALLOW_HALF_PRECISION_ACCUMULATION", "1")
    os.environ.setdefault("ONEFLOW_MATMUL_ALLOW_HALF_PRECISION_ACCUMULATION", "1")
    os.environ.setdefault("ONEFLOW_LINEAR_EMBEDDING_SKIP_INIT", "1")
    # os.environ.setdefault("ONEFLOW_ATTENTION_ALLOW_HALF_PRECISION_ACCUMULATION", "1")
    # os.environ.setdefault("ONEFLOW_ATTENTION_ALLOW_HALF_PRECISION_SCORE_ACCUMULATION_MAX_M", "-1")
    # os.environ.setdefault("ONEFLOW_ATTENTION_ALLOW_QUANTIZATION", "1")

    os.environ.setdefault("ONEFLOW_MLIR_GROUP_MATMUL_QUANT", "1")
    os.environ.setdefault("ONEFLOW_CONV2D_KERNEL_ENABLE_TUNING_WARMUP", "1")

    # TODO: enable this will cause the failure of multi resolution warmup
    # os.environ.setdefault("ONEFLOW_MLIR_FUSE_KERNEL_LAUNCH", "1")
    # os.environ.setdefault("ONEFLOW_KERNEL_ENABLE_CUDA_GRAPH", "1")


def set_nexfort_env_vars(options):
    field2env_var = {}
    _set_env_vars(field2env_var, options)


def set_nexfort_default_env_vars():
    pass


def set_default_env_vars():
    set_oneflow_default_env_vars()
    set_nexfort_default_env_vars()
